Unit Windows;            { J.Sklenar     December 1992 }
{$N+,E+}

Interface

Uses Crt,Graph;

(* This unit defines the objects PWINDOW and DWINDOW:

 They have the following PRIVATE attributes:
        Wx1,Wy1   :integer;   { Upper left corner }
        Wx2,Wy2   :integer;   { Bottom right corner }
        BdColor   :byte;      { Border color }
        BkColor   :byte;      { Background color }
        TColor    :byte;      { Text color }
        TEColor   :byte;      { Text emphasized color }
        P         :pointer;   { Pointer to memory }
        Size      :word;      { Memory size }
 They have these methods:
        WOpen  = open window
        WClose = close window
        WClear = clear window
        Wrt    = write to window
        WrtE   = write to window emphasized

 DWINDOW has a double border. It is a descendent of PWINDOW.
 It inherits all attributes and methods. The only difference
 is in methods WOpen and WClear to display double border.  *)

{************************* Object **************************}

Type  PWindowP  = ^PWindow;               { Pointers }
      DWindowP  = ^DWindow;

      PWindow = object
      {Methods:}
        Constructor WOpen(xul,yul,xbr,ybr:integer;
                               BdC,BkC,TC,TEC:byte);
        Procedure WClose;
        Procedure WClear;
        Procedure Wrt (xt,yt:integer; S:string);
        Procedure WrtE(xt,yt:integer; S:string);
      Private {attributes:}
        Wx1,Wy1   :integer;     { Upper left corner }
        Wx2,Wy2   :integer;   { Bottom right corner }
        BdColor   :byte;             { Border color }
        BkColor   :byte;         { Background color }
        TColor    :byte;               { Text color }
        TEColor   :byte;    { Text emphasized color }
        P         :pointer;     { Pointer to memory }
        Size      :word;              { Memory size }
      End;

      DWindow = object (PWindow)
      {Methods which are different:}
        Constructor WOpen(xul,yul,xbr,ybr:integer;
                               BdC,BkC,TC,TEC:byte);
        Procedure WClear;
      End;


{***********************************************************}

Implementation

Const    dw = 3;    { Difference between borders of DWindow }

{******************* Methods for PWindow *******************}

  Constructor PWindow.WOpen(xul,yul,xbr,ybr:integer;
                                    BdC,BkC,TC,TEC:byte);
  Begin
    { Store value attributes: }
    Wx1 := xul; Wy1 := yul;
    Wx2 := xbr; Wy2 := ybr;
    BdColor := BdC;
    BkColor := BkC;
    TColor  := TC;
    TEColor := TEC;
    { Get the size of RAM to save the screen contents: }
    Size := ImageSize(Wx1,Wy1,Wx2,Wy2);
    { Get the RAM from heap: }
    GetMem(P, Size);
    { Save the screen contents: }
    GetImage(Wx1,Wy1,Wx2,Wy2,P^);
    WClear;
  End;

{***********************************************************}

  Procedure PWindow.WClose;
  Begin
    { Restore the screen: }
    PutImage(Wx1,Wy1,P^,NormalPut);
    { Return RAM to heap: }
    FreeMem(P,Size);
  End;

{***********************************************************}

  Procedure PWindow.WClear;
  Begin
    { Display a bar to clear window: }
    SetFillStyle(SolidFill,BkColor);
    Bar(Wx1,Wy1,Wx2,Wy2);
    { Single border: }
    SetLineStyle(SolidLn,0,NormWidth);
    SetColor(BdColor);
    Rectangle(Wx1,Wy1,Wx2,Wy2);
  End;

{***********************************************************}

  Procedure PWindow.Wrt(xt,yt: integer; S: string);
  Begin
    SetColor(TColor);
    SetTextStyle(DefaultFont, HorizDir,1);
    SetTextJustify(LeftText,CenterText);
    OutTextXY(Wx1+8*xt,Wy1+16*yt,S);
  End;

{***********************************************************}

  Procedure PWindow.WrtE(xt,yt: integer; S: string);
  Begin
    SetColor(TEColor);
    SetTextStyle(DefaultFont, HorizDir,1);
    SetTextJustify(LeftText,CenterText);
    OutTextXY(Wx1+8*xt,Wy1+16*yt,S);
  End;

{******************* Methods for DWindow *******************}

  Constructor DWindow.WOpen(xul,yul,xbr,ybr:integer;
                                    BdC,BkC,TC,TEC:byte);
  Begin
    { Call ancestor's WOpen: }
    PWindow.WOpen(xul,yul,xbr,ybr,BdC,BkC,TC,TEC);
    WClear;
  End;

{***********************************************************}

  Procedure DWindow.WClear;
  Begin
    SetFillStyle(SolidFill,BkColor);
    Bar(Wx1,Wy1,Wx2,Wy2);
    { Double border: }
    SetLineStyle(SolidLn,0,NormWidth);
    SetColor(BdColor);
    Rectangle(Wx1,Wy1,Wx2,Wy2);
    Rectangle(Wx1+dw,Wy1+dw,Wx2-dw,Wy2-dw);
  End;

End.
